-------------------------------------------------------------------------------
--
-- Copyright 2021
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb2b_board_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE unb2b_board_lib.unb2b_board_pkg.ALL;

PACKAGE lofar2_unb2b_ring_pkg IS

 -----------------------------------------------------------------------------
  -- Revision control
  -----------------------------------------------------------------------------

  TYPE t_lofar2_unb2b_ring_config IS RECORD
    N_ring_lanes      : NATURAL; 
    N_error_counts    : NATURAL;
  END RECORD;

  CONSTANT c_one  : t_lofar2_unb2b_ring_config := (1, 8);
  CONSTANT c_full : t_lofar2_unb2b_ring_config := (8, 8);
  
  -- Function to select the revision configuration. 
  FUNCTION func_sel_revision_rec(g_design_name : STRING) RETURN t_lofar2_unb2b_ring_config;


END lofar2_unb2b_ring_pkg;


PACKAGE BODY lofar2_unb2b_ring_pkg IS

  FUNCTION func_sel_revision_rec(g_design_name : STRING) RETURN t_lofar2_unb2b_ring_config IS
  BEGIN
    IF    g_design_name = "lofar2_unb2b_ring_one"        THEN RETURN c_one;
    ELSE  RETURN c_full;
    END IF;
  END;


END lofar2_unb2b_ring_pkg;

