--------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb2c_board_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE unb2c_board_lib.unb2c_board_pkg.ALL;

PACKAGE lofar2_unb2c_filterbank_pkg IS

 -----------------------------------------------------------------------------
  -- Revision control
  -----------------------------------------------------------------------------

  TYPE t_lofar2_unb2c_filterbank_config IS RECORD
    nof_streams_jesd204b           : NATURAL;  
    nof_streams_db                 : NATURAL;  
    nof_streams_input              : NATURAL; 
    dp_clk_freq                    : NATURAL; 
  END RECORD;

  --                                                          nofjesd, nofdb, nofinput  
  CONSTANT c_full             : t_lofar2_unb2c_filterbank_config := (     12,     2,       12, c_unb2c_board_ext_clk_freq_200M );
  CONSTANT c_full_256MHz      : t_lofar2_unb2c_filterbank_config := (     12,     2,       12, c_unb2c_board_ext_clk_freq_256M );

  -- Function to select the revision configuration. 
  FUNCTION func_sel_revision_rec(g_design_name : STRING) RETURN t_lofar2_unb2c_filterbank_config;


END lofar2_unb2c_filterbank_pkg;


PACKAGE BODY lofar2_unb2c_filterbank_pkg IS

  FUNCTION func_sel_revision_rec(g_design_name : STRING) RETURN t_lofar2_unb2c_filterbank_config IS
  BEGIN
    IF    g_design_name = "lofar2_unb2c_filterbank_full"        THEN RETURN c_full;
    ELSIF g_design_name = "lofar2_unb2c_filterbank_full_256MHz" THEN RETURN c_full_256MHz;
    ELSE  RETURN c_full;
    END IF;
  END;


END lofar2_unb2c_filterbank_pkg;

