--------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb2c_board_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE unb2c_board_lib.unb2c_board_pkg.ALL;

PACKAGE lofar2_unb2c_sdp_station_pkg IS

 -----------------------------------------------------------------------------
  -- Revision control
  -----------------------------------------------------------------------------

  TYPE t_lofar2_unb2c_sdp_station_config IS RECORD
    no_jesd           : BOOLEAN;  
    use_fsub          : BOOLEAN; 
    use_bf            : BOOLEAN; 
    use_xsub          : BOOLEAN;
    use_ring          : BOOLEAN;
    P_sq              : NATURAL; 
  END RECORD;

  CONSTANT c_ait       : t_lofar2_unb2c_sdp_station_config := (FALSE, FALSE, FALSE, FALSE, FALSE, 0);
  CONSTANT c_fsub      : t_lofar2_unb2c_sdp_station_config := (FALSE, TRUE,  FALSE, FALSE, FALSE, 0);
  CONSTANT c_bf        : t_lofar2_unb2c_sdp_station_config := (FALSE, TRUE,  TRUE,  FALSE, FALSE, 0);
  CONSTANT c_bf_ring   : t_lofar2_unb2c_sdp_station_config := (FALSE, TRUE,  TRUE,  FALSE, TRUE,  0);
  CONSTANT c_xsub_one  : t_lofar2_unb2c_sdp_station_config := (FALSE, TRUE,  FALSE, TRUE,  FALSE, 1);
  CONSTANT c_xsub_ring : t_lofar2_unb2c_sdp_station_config := (FALSE, TRUE,  FALSE, TRUE,  TRUE,  9);
  CONSTANT c_full_wg   : t_lofar2_unb2c_sdp_station_config := (TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  9);
  CONSTANT c_full      : t_lofar2_unb2c_sdp_station_config := (FALSE, TRUE,  TRUE,  TRUE,  TRUE,  9);
  
  -- Function to select the revision configuration. 
  FUNCTION func_sel_revision_rec(g_design_name : STRING) RETURN t_lofar2_unb2c_sdp_station_config;


END lofar2_unb2c_sdp_station_pkg;


PACKAGE BODY lofar2_unb2c_sdp_station_pkg IS

  FUNCTION func_sel_revision_rec(g_design_name : STRING) RETURN t_lofar2_unb2c_sdp_station_config IS
  BEGIN
    IF    g_design_name = "lofar2_unb2c_sdp_station_adc"        THEN RETURN c_ait;
    ELSIF g_design_name = "lofar2_unb2c_sdp_station_fsub"       THEN RETURN c_fsub;
    ELSIF g_design_name = "lofar2_unb2c_sdp_station_bf"         THEN RETURN c_bf;
    ELSIF g_design_name = "lofar2_unb2c_sdp_station_bf_ring"    THEN RETURN c_bf_ring;
    ELSIF g_design_name = "lofar2_unb2c_sdp_station_xsub_one"   THEN RETURN c_xsub_one;
    ELSIF g_design_name = "lofar2_unb2c_sdp_station_xsub_ring"  THEN RETURN c_xsub_ring;
    ELSIF g_design_name = "lofar2_unb2c_sdp_station_full_wg"    THEN RETURN c_full_wg;
    ELSE  RETURN c_full;
    END IF;
  END;


END lofar2_unb2c_sdp_station_pkg;

