-------------------------------------------------------------------------------
--
-- Copyright 2021
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Author : R vd Walle, E. Kooistra
-- Purpose: Verify multiple variations of tb_sdp_statistics_offload
-- Description:
-- Usage:
-- > as 3
-- > run -all
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY tb_tb_sdp_statistics_offload IS
END tb_tb_sdp_statistics_offload;

ARCHITECTURE tb OF tb_tb_sdp_statistics_offload IS
  SIGNAL tb_end : STD_LOGIC := '0';  -- declare tb_end to avoid 'No objects found' error on 'when -label tb_end'
BEGIN

--    -- All
--    g_fast_mm_clk              : BOOLEAN := TRUE;  -- When TRUE use 1 GHz mm_clk  to speed up simulation, else use 100 MHz mm_clk
--                                                   -- for real speed of u_dp_block_from_mm_dc in sdp_statistics_offload
--    g_statistics_type          : STRING := "SST";
--    g_offload_time             : NATURAL := 500;
--    g_reverse_word_order       : BOOLEAN := TRUE  -- when TRUE then stream LSB word after MSB word.
--    g_gn_index                 : NATURAL := 1;  -- global node (GN) index, use > 0 to see effect of g_offload_time
--    g_nof_sync                 : NATURAL := 3;
--    -- BST
--    g_beamset_id               : NATURAL := 0;
--    -- XST
--    g_O_rn                     : NATURAL := 0;  -- GN index of first ring node (RN)
--    g_N_rn                     : NATURAL := 16; -- <= c_sdp_N_rn_max = 16, number of nodes in ring
--    g_P_sq                     : NATURAL := c_sdp_P_sq
--    g_nof_crosslets            : NATURAL := 1;
--    g_crosslets_direction      : INTEGER := 1;  -- +1 or -1

  u_sst                     : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "SST", 50,  TRUE, 3, 3);
  u_sst_no_reverse          : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "SST", 50, FALSE, 3, 3);
  u_bst_0                   : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "BST", 50,  TRUE, 1, 3);
  u_bst_0_no_reverse        : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "BST", 50, FALSE, 1, 3, 0);
  u_bst_1                   : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "BST", 50,  TRUE, 1, 3, 1);
  u_xst_P1                  : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 1, 3, 0, 0, 16,  1, 1, 1);
  u_xst_P1_N3               : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 1, 3, 0, 0, 16,  1, 3, 1);
  u_xst_P9                  : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 1, 3, 0, 0, 16,  9, 1, 1);
  u_xst_P9_N3               : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 1, 3, 0, 0, 16,  9, 3, 1);
  u_xst_P9_N3_no_reverse    : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50, FALSE, 1, 3, 0, 0, 16,  9, 3, 1);
  u_xst_P9_N3_neg_dir       : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 1, 3, 0, 0, 16,  9, 3, 0);
  u_xst_P8_N7_RN1_15        : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 1, 3, 0, 1, 15,  8, 7, 0);
  u_xst_P1_N7_RN0_7         : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 3, 3, 0, 0,  8,  1, 7, 1);  -- P_sq = 1 < N_rn/2+1 = 5
  u_xst_P9_N7_RN0_7         : ENTITY work.tb_sdp_statistics_offload GENERIC MAP( TRUE, "XST", 50,  TRUE, 3, 3, 0, 0,  8,  9, 7, 1);  -- P_sq = 9 > N_rn/2+1 = 5
  u_xst_P9_N4_RN0_7_slow_mm : ENTITY work.tb_sdp_statistics_offload GENERIC MAP(FALSE, "XST", 50,  TRUE, 3, 3, 0, 0,  8,  9, 4, 1);  -- P_sq = 9 > N_rn/2+1 = 5
  u_xst_P9_N7_RN0_7_slow_mm : ENTITY work.tb_sdp_statistics_offload GENERIC MAP(FALSE, "XST", 50,  TRUE, 3, 3, 0, 0,  8,  9, 7, 1);  -- P_sq = 9 > N_rn/2+1 = 5

END tb;
